package ktypes

// Int -- целочисленный тип ядра
type Int interface {
	// Get -- возвращает хранимое значение потокобезопасной переменной
	Get() int
	// Set -- устанавливает хранимое потокобезопасное значение
	Set(int)
	// Reset -- сбрасывает хранимое потокобезопасное значение
	Reset()
	// Inc -- увеличивает число на 1
	Inc()
	// Dec -- уменьшает число на 1
	Dec()
	// FromStr -- устанавливает значение из строки
	FromStr(string) Result[int]
	// ToStr -- возвращает как строку
	ToStr() string
	// FromByte -- устанавливает значение из байтов
	FromByte([]byte) Result[int]
	// ToByte -- возвращает как срез байт
	ToByte() []byte
	// FromBool -- устанавливает значение из булева
	FromBool(bool) int
	// ToBool -- возвращает как булево значение
	ToBool() bool
	// Getenv -- устанавливает значение из окружения
	Getenv(string) Result[int]
}
